@@ -35,7 +35,7 @@ SAFE_PERCENT_ENCODING = 'safe_percent_encoding'
 
 FILE_PATH = 'file_path'
 
-MAPPING_PARTITION = 'mapping_partition'
+MAPPING_PARTITIONING = 'mapping_partitioning'
 INFER_SQL_DATATYPES = 'infer_sql_datatypes'
 ENFORCE_SQL_QUERY_FILTER_NULL = 'enforce_sql_filter_null'
 
@@ -94,7 +94,7 @@ CONFIGURATION_OPTIONS_EMPTY_VALID = {
             SAFE_PERCENT_ENCODING: DEFAULT_SAFE_PERCENT_ENCODING,
             READ_PARSED_MAPPINGS_PATH: DEFAULT_READ_PARSED_MAPPINGS_PATH,
             WRITE_PARSED_MAPPINGS_PATH: DEFAULT_WRITE_PARSED_MAPPINGS_PATH,
-            MAPPING_PARTITION: PARTIAL_AGGREGATIONS_PARTITIONING,
+            MAPPING_PARTITIONING: PARTIAL_AGGREGATIONS_PARTITIONING,
             LOGGING_FILE: DEFAULT_LOGGING_FILE,
             ORACLE_CLIENT_LIB_DIR: DEFAULT_ORACLE_CLIENT_LIB_DIR,
             ORACLE_CLIENT_CONFIG_DIR: DEFAULT_ORACLE_CLIENT_LIB_DIR,
@@ -173,12 +173,12 @@ class Config(ConfigParser):
                              f'It must be in: {VALID_LOGGING_LEVEL}.')
 
         # MAPPING PARTITIONING
-        mapping_partitioning = str(self.get_mapping_partition()).upper()
-        self.set_mapping_partition(mapping_partitioning)
+        mapping_partitioning = str(self.get_mapping_partitioning()).upper()
+        self.set_mapping_partitioning(mapping_partitioning)
         if mapping_partitioning not in NO_PARTITIONING + [PARTIAL_AGGREGATIONS_PARTITIONING] + [
                 MAXIMAL_PARTITIONING]:
             raise ValueError(
-                f'{MAPPING_PARTITION} value `{self.get_mapping_partition()}` is not valid. '
+                f'{MAPPING_PARTITIONING} value `{self.get_mapping_partitioning()}` is not valid. '
                 f'It must be in: {[MAXIMAL_PARTITIONING] + [PARTIAL_AGGREGATIONS_PARTITIONING] + NO_PARTITIONING}.')
 
     def log_config_info(self):
@@ -245,8 +245,8 @@ class Config(ConfigParser):
     def get_oracle_client_config_dir(self):
         return self.get(self.configuration_section, ORACLE_CLIENT_CONFIG_DIR)
 
-    def get_mapping_partition(self):
-        return self.get(self.configuration_section, MAPPING_PARTITION)
+    def get_mapping_partitioning(self):
+        return self.get(self.configuration_section, MAPPING_PARTITIONING)
 
     def get_output_dir(self):
         return self.get(self.configuration_section, OUTPUT_DIR)
@@ -280,8 +280,8 @@ class Config(ConfigParser):
 
         return file_path.as_posix()
 
-    def set_mapping_partition(self, mapping_partitioning):
-        self.set(self.configuration_section, MAPPING_PARTITION, mapping_partitioning.upper())
+    def set_mapping_partitioning(self, mapping_partitioning):
+        self.set(self.configuration_section, MAPPING_PARTITIONING, mapping_partitioning.upper())
 
     def set_logging_level(self, logging_level):
         self.set(self.configuration_section, LOGGING_LEVEL, logging_level)
@@ -289,6 +289,9 @@ class Config(ConfigParser):
     def set_output_format(self, output_format):
         self.set(self.configuration_section, OUTPUT_FORMAT, output_format)
 
+    def set_number_of_processes(self, number_of_processes):
+        self.set(self.configuration_section, NUMBER_OF_PROCESSES, number_of_processes)
+
     ################################################################################
     #######################   DATA SOURCE SECTIONS METHODS   #######################
     ################################################################################
@@ -315,7 +318,7 @@ class Config(ConfigParser):
                     mapping_file = os.path.join(mapping_path, mapping_file_name)
                     if os.path.isfile(mapping_file):
                         mapping_file_paths.append(mapping_file)
-            # if it is an URL
+            # if it is a URL
             elif mapping_path.startswith('http'):
                 mapping_file_paths.append(mapping_path)
             else:
