@@ -15,12 +15,21 @@ MAXIMAL_PARTITIONING = 'MAXIMAL'
 NO_PARTITIONING = ['NO', 'FALSE', 'OFF', '0']
 
 
+##############################################################################
+########################   MAPPING PARTITION OPTIONS   #######################
+##############################################################################
+
+PARTIAL_AGGREGATIONS_PARTITIONING = 'PARTIAL-AGGREGATIONS'
+MAXIMAL_PARTITIONING = 'MAXIMAL'
+NO_PARTITIONING = ['NO', 'FALSE', 'OFF', '0']
+
+
 ##############################################################################
 #########################   DATA SOURCE TYPES   ##############################
 ##############################################################################
 
 # data files
-RDB = 'RDB'
+TV = 'TV'   # RML Tabular View
 CSV = 'CSV'
 TSV = 'TSV'
 EXCEL = ['XLS', 'XLSX', 'XLSM', 'XLSB']
@@ -35,6 +44,7 @@ JSON = 'JSON'
 XML = 'XML'
 
 # DBMSs
+RDB = 'RDB'
 MYSQL = 'MYSQL'
 MARIADB = 'MARIADB'
 MSSQL = 'MSSQL'
@@ -42,7 +52,7 @@ ORACLE = 'ORACLE'
 POSTGRESQL = 'POSTGRESQL'
 SQLITE = 'SQLITE'
 
-FILE_SOURCE_TYPES = [CSV, TSV, PARQUET, ORC, STATA, SPSS, JSON, XML] + EXCEL + FEATHER + SAS + ODS
+FILE_SOURCE_TYPES = [TV, CSV, TSV, PARQUET, ORC, STATA, SPSS, JSON, XML] + EXCEL + FEATHER + SAS + ODS
 DATA_SOURCE_TYPES = [RDB] + FILE_SOURCE_TYPES
 
 # RDF serializations
@@ -130,6 +140,7 @@ RML_NAMESPACE = 'http://semweb.mmlab.be/ns/rml#'
 
 # properties
 RML_LOGICAL_SOURCE = f'{RML_NAMESPACE}logicalSource'
+RML_SOURCE = f'{RML_NAMESPACE}source'
 RML_QUERY = f'{RML_NAMESPACE}query'
 RML_ITERATOR = f'{RML_NAMESPACE}iterator'
 RML_REFERENCE = f'{RML_NAMESPACE}reference'
@@ -154,6 +165,8 @@ RML_STAR_NON_ASSERTED_TRIPLES_MAP_CLASS = f'{RML_NAMESPACE}NonAssertedTriplesMap
 RML_STAR_QUOTED_TRIPLES_MAP = f'{RML_NAMESPACE}quotedTriplesMap'
 RML_STAR_SUBJECT_MAP = f'{RML_NAMESPACE}subjectMap'
 RML_STAR_OBJECT_MAP = f'{RML_NAMESPACE}objectMap'
+RML_STAR_SUBJECT_CONSTANT_SHORTCUT = f'{RML_NAMESPACE}subject'
+RML_STAR_OBJECT_CONSTANT_SHORTCUT = f'{RML_NAMESPACE}object'
 
 # other
 RML_STAR_RDF_STAR_TRIPLE = f'{RML_NAMESPACE}RDFstarTriple'
@@ -176,17 +189,11 @@ XSD_DATETIME = f'{XSD_NAMESPACE}dateTime'
 XSD_STRING = f'{XSD_NAMESPACE}string'
 
 
-##############################################################################
-#############################   XSD DATA TYPES   #############################
-##############################################################################
-
-RDF_NAMESPACE = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'
-
-RDF_TYPE = f'{RDF_NAMESPACE}type'
-
-
 ##############################################################################
 ##################################   OTHER   #################################
 ##############################################################################
 
+RDF_NAMESPACE = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'
+RDF_TYPE = f'{RDF_NAMESPACE}type'
+
 AUXILIAR_UNIQUE_REPLACING_STRING = 'zzyy_xxww\u200B'
