@@ -7,12 +7,12 @@ __email__ = "arenas.guerrero.julian@outlook.com"
 
 
 import json
+import duckdb
 import pandas as pd
 import numpy as np
 import elementpath
 import xml.etree.ElementTree as et
 import urllib.request
-import os
 
 from jsonpath import JSONPath
 from elementpath.xpath3 import XPath3Parser
@@ -25,7 +25,9 @@ def get_file_data(mapping_rule, references):
     references = list(references)
     file_source_type = mapping_rule['source_type']
 
-    if file_source_type in [CSV, TSV]:
+    if file_source_type == TV:
+        return _read_tabular_view(mapping_rule)
+    elif file_source_type in [CSV, TSV]:
         return _read_csv(mapping_rule, references, file_source_type)
     elif file_source_type in EXCEL:
         return _read_excel(mapping_rule, references)
@@ -51,35 +53,52 @@ def get_file_data(mapping_rule, references):
         raise ValueError(f'Found an invalid source type. Found value `{file_source_type}`.')
 
 
+def _read_tabular_view(mapping_rule):
+    return duckdb.query(mapping_rule['logical_source_value']).df()
+
+
 def _read_csv(mapping_rule, references, file_source_type):
     delimiter = ',' if file_source_type == 'CSV' else '\t'
 
-    return pd.read_table(mapping_rule['data_source'],
-                         delimiter=delimiter,
-                         index_col=False,
-                         encoding='utf-8',
-                         encoding_errors='strict',
-                         usecols=references,
-                         engine='c',
-                         dtype=str,
-                         keep_default_na=False,
-                         na_filter=False)
+    try:
+        return pd.read_table(mapping_rule['logical_source_value'],
+                             sep=delimiter,
+                             index_col=False,
+                             encoding='utf-8',
+                             encoding_errors='strict',
+                             usecols=references,
+                             engine='c',
+                             dtype=str,
+                             keep_default_na=False,
+                             na_filter=False)
+    except:
+        # if delimiter is other than comma or tab, then infer it (issue #81)
+        return pd.read_table(mapping_rule['logical_source_value'],
+                             index_col=False,
+                             sep=None,
+                             encoding='utf-8',
+                             encoding_errors='strict',
+                             usecols=references,
+                             engine='python',
+                             dtype=str,
+                             keep_default_na=False,
+                             na_filter=False)
 
 
 def _read_parquet(mapping_rule, references):
-    return pd.read_parquet(mapping_rule['data_source'], engine='pyarrow', columns=references)
+    return pd.read_parquet(mapping_rule['logical_source_value'], engine='pyarrow', columns=references)
 
 
 def _read_feather(mapping_rule, references):
-    return pd.read_feather(mapping_rule['data_source'], use_threads=False, columns=references)
+    return pd.read_feather(mapping_rule['logical_source_value'], use_threads=False, columns=references)
 
 
 def _read_orc(mapping_rule, references):
-    return pd.read_orc(mapping_rule['data_source'], encoding='utf-8', columns=references)
+    return pd.read_orc(mapping_rule['logical_source_value'], encoding='utf-8', columns=references)
 
 
 def _read_stata(mapping_rule, references):
-    return pd.read_stata(mapping_rule['data_source'],
+    return pd.read_stata(mapping_rule['logical_source_value'],
                          columns=references,
                          convert_dates=False,
                          convert_categoricals=False,
@@ -89,15 +108,15 @@ def _read_stata(mapping_rule, references):
 
 
 def _read_sas(mapping_rule):
-    return pd.read_sas(mapping_rule['data_source'], encoding='utf-8')
+    return pd.read_sas(mapping_rule['logical_source_value'], encoding='utf-8')
 
 
 def _read_spss(mapping_rule, references):
-    return pd.read_spss(mapping_rule['data_source'], usecols=references, convert_categoricals=False)
+    return pd.read_spss(mapping_rule['logical_source_value'], usecols=references, convert_categoricals=False)
 
 
 def _read_excel(mapping_rule, references):
-    return pd.read_excel(mapping_rule['data_source'],
+    return pd.read_excel(mapping_rule['logical_source_value'],
                          sheet_name=0,
                          engine='openpyxl',
                          usecols=references,
@@ -107,7 +126,7 @@ def _read_excel(mapping_rule, references):
 
 
 def _read_ods(mapping_rule, references):
-    return pd.read_excel(mapping_rule['data_source'],
+    return pd.read_excel(mapping_rule['logical_source_value'],
                          sheet_name=0,
                          engine='odf',
                          usecols=references,
@@ -117,17 +136,17 @@ def _read_ods(mapping_rule, references):
 
 
 def _read_json(mapping_rule, references):
-    if os.path.isfile(mapping_rule['data_source']):
-        with open(mapping_rule['data_source'], encoding='utf-8') as json_file:
-            json_data = json.load(json_file)
-    else:
-        with urllib.request.urlopen(mapping_rule['data_source']) as json_url:
+    if mapping_rule['logical_source_value'].startswith('http'):
+        with urllib.request.urlopen(mapping_rule['logical_source_value']) as json_url:
             json_data = json.loads(json_url.read().decode())
+    else:
+        with open(mapping_rule['logical_source_value'], encoding='utf-8') as json_file:
+            json_data = json.load(json_file)
 
     jsonpath_expression = mapping_rule['iterator'] + '.('
     # add top level object of the references to reduce intermediate results (THIS IS NOT STRICTLY NECESSARY)
     for reference in references:
-        jsonpath_expression += reference.split('.')[0] + ','
+        jsonpath_expression += reference + ','
     jsonpath_expression = jsonpath_expression[:-1] + ')'
 
     jsonpath_result = JSONPath(jsonpath_expression).parse(json_data)
@@ -142,12 +161,12 @@ def _read_json(mapping_rule, references):
 
 
 def _read_xml(mapping_rule, references):
-    if os.path.isfile(mapping_rule['data_source']):
-        with open(mapping_rule['data_source'], encoding='utf-8') as xml_file:
-            xml_root = et.parse(xml_file).getroot()
-    else:
-        with urllib.request.urlopen(mapping_rule['data_source']) as xml_url:
+    if mapping_rule['logical_source_value'].startswith('http'):
+        with urllib.request.urlopen(mapping_rule['logical_source_value']) as xml_url:
             xml_root = et.ElementTree(et.fromstring(xml_url.read().decode())).getroot()
+    else:
+        with open(mapping_rule['logical_source_value'], encoding='utf-8') as xml_file:
+            xml_root = et.parse(xml_file).getroot()
 
     xpath_result = elementpath.iter_select(xml_root, mapping_rule['iterator'], parser=XPath3Parser)
 
