@@ -72,7 +72,7 @@ def _generate_maximal_partition_for_a_position_ordering(mappings_df, position_or
             mappings_df.sort_values(by=['mapping_partition', 'predicate_invariant'], inplace=True, ascending=True)
 
             # if all predicates are constant terms we can use full string comparison instead of startswith
-            enforce_invariant_non_subset = mappings_df['predicate_constant'].notna().all()
+            enforce_invariant_non_subset = set(mappings_df['predicate_map_value']) == set(R2RML_CONSTANT)
 
             for i, mapping_rule in mappings_df.iterrows():
                 if current_global_group != mapping_rule['mapping_partition']:
@@ -123,7 +123,7 @@ def _generate_maximal_partition_for_a_position_ordering(mappings_df, position_or
             mappings_df.sort_values(by=['mapping_partition', 'graph_invariant'], inplace=True, ascending=True)
 
             # if all graph are constant terms we can use full string comparison instead of startswith
-            enforce_invariant_non_subset = mappings_df['graph_constant'].notna().all()
+            enforce_invariant_non_subset = set(mappings_df['graph_map_value']) == set(R2RML_CONSTANT)
 
             for i, mapping_rule in mappings_df.iterrows():
                 if current_global_group != mapping_rule['mapping_partition']:
@@ -168,20 +168,22 @@ class MappingPartitioner:
         self.mappings_df = self.mappings_df.reset_index(drop=True)
 
         # if RML-star or TMs without POMs (rml:NonAssertedTriplesMap) do not partition mappings (assign empty partition)
-        if self.mappings_df['subject_quoted'].notnull().any() or self.mappings_df['object_quoted'].notnull().any() or \
-                RML_STAR_NON_ASSERTED_TRIPLES_MAP_CLASS in set(self.mappings_df['triples_map_type']):
+        if RML_STAR_QUOTED_TRIPLES_MAP in self.mappings_df['subject_map_type'].values or \
+            RML_STAR_QUOTED_TRIPLES_MAP in self.mappings_df['object_map_type'] or \
+            RML_STAR_NON_ASSERTED_TRIPLES_MAP_CLASS in set(self.mappings_df['triples_map_type']):
+
             # TODO: enable mapping partitioning for these cases
             self.mappings_df['mapping_partition'] = '0-0-0-0'
 
             return self.mappings_df
 
-        if self.config.get_mapping_partition() == PARTIAL_AGGREGATIONS_PARTITIONING:
+        if self.config.get_mapping_partitioning() == PARTIAL_AGGREGATIONS_PARTITIONING:
             self._get_term_invariants()
             self._generate_partial_aggregations_partition()
-        elif self.config.get_mapping_partition() == MAXIMAL_PARTITIONING:
+        elif self.config.get_mapping_partitioning() == MAXIMAL_PARTITIONING:
             self._get_term_invariants()
             self._generate_maximal_partition()
-        elif self.config.get_mapping_partition() in NO_PARTITIONING:
+        elif self.config.get_mapping_partitioning() in NO_PARTITIONING:
             # assign empty partition
             self.mappings_df['mapping_partition'] = '0-0-0-0'
         else:
@@ -203,11 +205,6 @@ class MappingPartitioner:
 
         mappings_df = self.mappings_df.copy()
 
-        if mappings_df['predicate_constant'].notna().all():
-            logging.debug('All predicate maps are constant-valued, invariant subset is not enforced.')
-        if mappings_df['graph_constant'].notna().all():
-            logging.debug('All graph maps are constant-valued, invariant subset is not enforced.')
-
         position_orderings = list(permutations(['S', 'P', 'O', 'G']))
 
         if self.config.is_multiprocessing_enabled():
@@ -287,7 +284,8 @@ class MappingPartitioner:
         current_invariant = AUXILIAR_UNIQUE_REPLACING_STRING
 
         # if all predicates are constant terms we can use full string comparison instead of startswith
-        enforce_invariant_non_subset = self.mappings_df['predicate_constant'].notna().all()
+        enforce_invariant_non_subset = set(self.mappings_df['predicate_map_value']) == set(R2RML_CONSTANT)
+
         if enforce_invariant_non_subset:
             logging.debug('All predicate maps are constant-valued, invariant subset is not enforced.')
 
@@ -334,7 +332,8 @@ class MappingPartitioner:
         current_invariant = AUXILIAR_UNIQUE_REPLACING_STRING
 
         # if all graph are constant terms we can use full string comparison instead of startswith
-        enforce_invariant_non_subset = self.mappings_df['graph_constant'].notna().all()
+        enforce_invariant_non_subset = set(self.mappings_df['graph_map_value']) == set(R2RML_CONSTANT)
+
         if enforce_invariant_non_subset:
             logging.debug('All graph maps are constant-valued, invariant subset is not enforced.')
 
@@ -382,37 +381,41 @@ class MappingPartitioner:
 
         for i, mapping_rule in self.mappings_df.iterrows():
             # SUBJECT
-            if pd.notna(mapping_rule['subject_template']):
+            if mapping_rule['subject_map_type'] == R2RML_TEMPLATE:
                 self.mappings_df.at[i, 'subject_invariant'] = \
-                    get_invariant_of_template(str(mapping_rule['subject_template']))
-            elif pd.notna(mapping_rule['subject_constant']):
-                self.mappings_df.at[i, 'subject_invariant'] = str(mapping_rule['subject_constant'])
-
+                    get_invariant_of_template(str(mapping_rule['subject_map_value']))
+            elif mapping_rule['subject_map_type'] == R2RML_CONSTANT:
+                self.mappings_df.at[i, 'subject_invariant'] = str(mapping_rule['subject_map_value'])
             # PREDICATE
-            if pd.notna(mapping_rule['predicate_constant']):
-                self.mappings_df.at[i, 'predicate_invariant'] = str(mapping_rule['predicate_constant'])
-            elif pd.notna(mapping_rule['predicate_template']):
+            if mapping_rule['predicate_map_type'] == R2RML_CONSTANT:
+                self.mappings_df.at[i, 'predicate_invariant'] = str(mapping_rule['predicate_map_value'])
+            elif mapping_rule['predicate_map_type'] == R2RML_TEMPLATE:
                 self.mappings_df.at[i, 'predicate_invariant'] = \
-                    get_invariant_of_template(str(mapping_rule['predicate_template']))
+                    get_invariant_of_template(str(mapping_rule['predicate_map_value']))
 
             # OBJECT
-            if pd.notna(mapping_rule['object_constant']):
-                self.mappings_df.at[i, 'object_invariant'] = str(mapping_rule['object_constant'])
-            elif pd.notna(mapping_rule['object_template']):
+            if mapping_rule['object_map_type'] == R2RML_CONSTANT:
+                self.mappings_df.at[i, 'object_invariant'] = str(mapping_rule['object_map_value'])
+            elif mapping_rule['object_map_type'] == R2RML_TEMPLATE:
                 self.mappings_df.at[i, 'object_invariant'] = \
-                    get_invariant_of_template(str(mapping_rule['object_template']))
-            elif pd.notna(mapping_rule['object_parent_triples_map']):
+                    get_invariant_of_template(str(mapping_rule['object_map_value']))
+            # elif pd.notna(mapping_rule['object_parent_triples_map']) and mapping_rule['object_parent_triples_map']!="":
+            elif mapping_rule['object_map_type'] == R2RML_PARENT_TRIPLES_MAP:
+
                 # get the invariant for referencing object maps
-                parent_mapping_rule = get_mapping_rule(self.mappings_df, mapping_rule['object_parent_triples_map'])
-                if pd.notna(parent_mapping_rule['subject_constant']):
-                    self.mappings_df.at[i, 'object_invariant'] = str(parent_mapping_rule['subject_constant'])
-                elif pd.notna(parent_mapping_rule['subject_template']):
+                # parent_mapping_rule = get_mapping_rule(self.mappings_df, mapping_rule['object_parent_triples_map'])
+                parent_mapping_rule = get_mapping_rule(self.mappings_df, mapping_rule['object_map_value'])
+
+
+                if mapping_rule['subject_map_type'] == R2RML_CONSTANT:
+                    self.mappings_df.at[i, 'object_invariant'] = str(parent_mapping_rule['subject_map_value'])
+                elif mapping_rule['subject_map_type'] == R2RML_TEMPLATE:
                     self.mappings_df.at[i, 'object_invariant'] = \
-                        get_invariant_of_template(str(parent_mapping_rule['subject_template']))
+                        get_invariant_of_template(str(parent_mapping_rule['subject_map_value']))
 
             # GRAPH
-            if pd.notna(mapping_rule['graph_constant']):
-                self.mappings_df.at[i, 'graph_invariant'] = str(mapping_rule['graph_constant'])
-            elif pd.notna(mapping_rule['graph_template']):
+            if mapping_rule['graph_map_type'] == R2RML_CONSTANT:
+                self.mappings_df.at[i, 'graph_invariant'] = str(mapping_rule['graph_map_value'])
+            elif mapping_rule['graph_map_type'] == R2RML_TEMPLATE:
                 self.mappings_df.at[i, 'graph_invariant'] = \
-                    get_invariant_of_template(str(mapping_rule['graph_template']))
+                    get_invariant_of_template(str(mapping_rule['graph_map_value']))
